import * as React from 'react';
import { Text, TouchableOpacity, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import MaterialIcon from 'react-native-vector-icons/MaterialIcons';
import { withTheme } from '../services/ThemeContext';
class UserListItemComponent extends React.Component {
    constructor() {
        super(...arguments);
        this.handleDelete = () => {
            this.props.onDelete(this.props.data);
        };
        this.handlePress = () => {
            this.props.onClick(this.props.data);
        };
    }
    render() {
        return (React.createElement(TouchableOpacity, { onPress: this.handlePress }, this.renderInside()));
    }
    renderInside() {
        const { theme } = this.props;
        const styles = getStyles(theme);
        return (React.createElement(View, { style: styles.container },
            React.createElement(View, { style: styles.textComtainer },
                React.createElement(Text, { style: styles.text }, this.props.data)),
            React.createElement(TouchableOpacity, { style: styles.iconButtonContainer, onPress: this.handleDelete },
                React.createElement(MaterialIcon, { style: styles.iconButtonIcon, name: "close", size: theme.rem(1) }))));
    }
}
export const UserListItem = withTheme(UserListItemComponent);
const getStyles = cacheStyles((theme) => ({
    container: {
        height: theme.rem(2.5),
        width: '100%',
        backgroundColor: theme.modal,
        flexDirection: 'row',
        alignItems: 'center'
    },
    textComtainer: {
        flex: 25,
        height: '100%',
        flexDirection: 'column',
        justifyContent: 'space-around'
    },
    iconButtonContainer: {
        flex: 5,
        flexDirection: 'row',
        alignItems: 'center',
        justifyContent: 'space-around',
        height: '100%'
    },
    iconButtonIcon: {
        color: theme.icon
    },
    text: {
        paddingLeft: theme.rem(1.25),
        color: theme.primaryText,
        backgroundColor: '#fff0',
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(1)
    }
}));
