var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Keyboard, ScrollView, View } from 'react-native';
import { cacheStyles } from 'react-native-patina';
import { submitLogin } from '../../actions/LoginCompleteActions';
import { maybeRouteComplete } from '../../actions/LoginInitActions';
import s from '../../common/locales/strings';
import { useHandler } from '../../hooks/useHandler.js';
import { useImports } from '../../hooks/useImports';
import { useScrollToEnd } from '../../hooks/useScrollToEnd';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { logEvent } from '../../util/analytics';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { DigitInput, MAX_PIN_LENGTH } from '../themed/DigitInput';
import { EdgeText } from '../themed/EdgeText';
import { Fade } from '../themed/Fade';
import { MainButton } from '../themed/MainButton';
import { ThemedScene } from '../themed/ThemedScene';
const ChangePinSceneComponent = ({ title, onBack, onSkip, onSubmit, mainButtonLabel = s.strings.done }) => {
    const theme = useTheme();
    const styles = getStyles(theme);
    const pin = useSelector(state => { var _a; return (_a = state.create.pin) !== null && _a !== void 0 ? _a : ''; });
    const pinErrorMessage = useSelector(state => { var _a; return (_a = state.create.pinErrorMessage) !== null && _a !== void 0 ? _a : ''; });
    const isValidPin = pin.length === MAX_PIN_LENGTH && pinErrorMessage === '';
    const handlePress = useHandler(() => (isValidPin ? onSubmit(pin) : undefined));
    const scrollViewRef = useScrollToEnd(isValidPin);
    return (React.createElement(ThemedScene, { onBack: onBack, onSkip: onSkip, title: title },
        React.createElement(ScrollView, { ref: scrollViewRef, style: styles.content },
            React.createElement(EdgeText, { style: styles.description, numberOfLines: 0 }, s.strings.pin_desc),
            React.createElement(DigitInput, null),
            React.createElement(View, { style: styles.actions },
                React.createElement(Fade, { visible: isValidPin },
                    React.createElement(MainButton, { label: mainButtonLabel, type: "secondary", onPress: handlePress }))))));
};
const getStyles = cacheStyles((theme) => ({
    content: {
        flex: 1,
        marginHorizontal: theme.rem(0.5),
        marginTop: theme.rem(1.5)
    },
    description: {
        fontFamily: theme.fontFaceDefault,
        fontSize: theme.rem(0.875),
        marginBottom: theme.rem(3.25)
    },
    actions: {
        flexDirection: 'row',
        justifyContent: 'center',
        marginTop: theme.rem(5),
        minHeight: theme.rem(3 + 15) // 15 is a hack to avoid the keyboard
    }
}));
// The scene for existing users to change their PIN
export const ChangePinScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const { onComplete } = useImports();
    const handleSubmit = useHandler((pin) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        try {
            yield account.changePin({ pin });
            yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: s.strings.pin_changed, message: s.strings.pin_successfully_changed, buttons: { ok: { label: s.strings.ok } } })));
            onComplete();
        }
        catch (e) {
            showError(e);
        }
    }));
    return React.createElement(ChangePinSceneComponent, { onSubmit: handleSubmit });
};
// The scene for new users to recover their PIN
export const ResecurePinScene = (props) => {
    const { route } = props;
    const { account } = route.params;
    const { onComplete, onLogin } = useImports();
    const dispatch = useDispatch();
    const handleComplete = () => {
        if (onLogin != null)
            dispatch(submitLogin(account));
        else
            onComplete();
    };
    const handleSubmit = useHandler((pin) => __awaiter(void 0, void 0, void 0, function* () {
        Keyboard.dismiss();
        try {
            yield account.changePin({ pin });
            yield Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: s.strings.pin_changed, message: s.strings.pin_successfully_changed, buttons: { ok: { label: s.strings.ok } } })));
            handleComplete();
        }
        catch (e) {
            showError(e);
        }
    }));
    return (React.createElement(ChangePinSceneComponent, { onSkip: handleComplete, title: s.strings.change_pin, onSubmit: handleSubmit }));
};
// The scene for new users to set their PIN
export const NewAccountPinScene = (props) => {
    const dispatch = useDispatch();
    const handleBack = useHandler(() => {
        dispatch(maybeRouteComplete({
            type: 'NAVIGATE',
            data: { name: 'newAccountPassword', params: {} }
        }));
    });
    const handleSubmit = useHandler(() => {
        logEvent('Signup_PIN_Valid');
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'newAccountTos', params: {} }
        });
    });
    return (React.createElement(ChangePinSceneComponent, { title: s.strings.choose_title_pin, onBack: handleBack, onSubmit: handleSubmit, mainButtonLabel: s.strings.next_label }));
};
