var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { Keyboard, ScrollView, TouchableOpacity, TouchableWithoutFeedback, View } from 'react-native';
import { KeyboardAwareScrollView } from 'react-native-keyboard-aware-scroll-view';
import { cacheStyles } from 'react-native-patina';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import MaterialIcon from 'react-native-vector-icons/MaterialIcons';
import { sprintf } from 'sprintf-js';
import { launchPasswordRecovery, login } from '../../actions/LoginAction';
import { maybeRouteComplete } from '../../actions/LoginInitActions';
import { deleteUserFromDevice } from '../../actions/UserActions';
import s from '../../common/locales/strings';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { logEvent } from '../../util/analytics';
import { base58 } from '../../util/base58';
import { LogoImageHeader } from '../abSpecific/LogoImageHeader';
import { UserListItem } from '../abSpecific/UserListItem';
import { ButtonsModal } from '../modals/ButtonsModal';
import { showQrCodeModal } from '../modals/QrCodeModal';
import { TextInputModal } from '../modals/TextInputModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { LineFormField } from '../themed/LineFormField';
import { MainButton } from '../themed/MainButton';
import { ThemedScene } from '../themed/ThemedScene';
class PasswordLoginSceneComponent extends React.Component {
    constructor(props) {
        super(props);
        this.handlePasswordChange = (password) => {
            this.setState({ errorMessage: '', password });
        };
        this.handleSubmit = () => __awaiter(this, void 0, void 0, function* () {
            const { login, saveOtpError, username } = this.props;
            const { password } = this.state;
            this.handleBlur();
            Keyboard.dismiss();
            const attempt = { type: 'password', username, password };
            yield login(attempt).catch(error => {
                if (error != null && error.name === 'OtpError') {
                    saveOtpError(attempt, error);
                }
                else {
                    console.log(error);
                    const errorMessage = error != null ? error.message : '';
                    this.setState({ errorMessage });
                }
            });
        });
        this.handleBack = () => {
            this.props.exitScene();
        };
        this.handleBlur = () => {
            Keyboard.dismiss();
            this.setState({
                focusFirst: false,
                focusSecond: false
            });
        };
        this.handleDelete = (username) => {
            const { deleteUserFromDevice } = this.props;
            Keyboard.dismiss();
            Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: s.strings.forget_account, message: sprintf(s.strings.forget_username_account, username), buttons: {
                    ok: { label: s.strings.forget },
                    cancel: { label: s.strings.cancel, type: 'secondary' }
                } })))
                .then((button) => __awaiter(this, void 0, void 0, function* () {
                if (button !== 'ok')
                    return;
                return yield deleteUserFromDevice(username);
            }))
                .catch(showError);
        };
        this.handleToggleUsernameList = () => {
            Keyboard.dismiss();
            this.setState({
                focusFirst: false,
                focusSecond: false,
                usernameList: !this.state.usernameList
            });
        };
        this.handleFocus1 = () => {
            this.setState({
                focusFirst: true,
                focusSecond: false
            });
        };
        this.handleFocus2 = () => {
            this.setState({
                focusFirst: false,
                focusSecond: true
            });
        };
        this.handleSetNextFocus = () => {
            this.setState({
                focusFirst: false,
                focusSecond: true
            });
        };
        this.handleSelectUser = (username) => {
            this.handleChangeUsername(username);
            this.setState({
                usernameList: false
            });
            const details = this.props.previousUsers.find(info => info.username === username);
            if (details != null &&
                (details.pinEnabled || (details.touchEnabled && this.props.touch))) {
                this.props.gotoPinLoginPage();
                return;
            }
            this.handleSetNextFocus();
        };
        this.handleChangeUsername = (data) => {
            this.setState({ errorMessage: '' });
            this.props.updateUsername(data);
        };
        this.handleForgotPassword = () => {
            Keyboard.dismiss();
            logEvent('Login_Password_Forgot_Password');
            Airship.show(bridge => (React.createElement(TextInputModal, { bridge: bridge, onSubmit: this.props.handleSubmitRecoveryKey, title: s.strings.password_recovery, message: s.strings.initiate_password_recovery, inputLabel: s.strings.recovery_token })));
        };
        this.handleCreateAccount = () => {
            logEvent('Login_Password_Create_Account');
            this.props.gotoCreatePage();
        };
        this.state = {
            errorMessage: '',
            focusFirst: true,
            focusSecond: false,
            password: '',
            usernameList: false
        };
    }
    render() {
        const { theme } = this.props;
        const styles = getStyles(theme);
        return (React.createElement(KeyboardAwareScrollView, { style: styles.container, keyboardShouldPersistTaps: "always", contentContainerStyle: styles.mainScrollView },
            React.createElement(TouchableWithoutFeedback, { onPress: this.handleBlur },
                React.createElement(ThemedScene, { onBack: this.handleBack, noUnderline: true, branding: this.props.branding }, this.renderOverImage()))));
    }
    renderOverImage() {
        const { theme } = this.props;
        const styles = getStyles(theme);
        if (this.props.loginSuccess) {
            /* return (
              <View style={style.featureBox}>
                <Text>LOGIN SUCCESS</Text>
              </View>
            ) */
            return null;
        }
        return (React.createElement(View, { style: styles.featureBoxContainer },
            React.createElement(TouchableWithoutFeedback, { onPress: this.handleBlur },
                React.createElement(View, { style: styles.featureBox },
                    React.createElement(LogoImageHeader, { branding: this.props.branding }),
                    this.renderUsername(),
                    React.createElement(View, { style: styles.shimTiny }),
                    React.createElement(LineFormField, { testID: "passwordFormField", onChangeText: this.handlePasswordChange, value: this.state.password, label: s.strings.password, error: this.state.errorMessage, autoCorrect: false, secureTextEntry: true, returnKeyType: "go", forceFocus: this.state.focusSecond, onFocus: this.handleFocus2, onSubmitEditing: this.handleSubmit }),
                    this.renderButtons()))));
    }
    renderUsername() {
        const { theme } = this.props;
        const styles = getStyles(theme);
        return (React.createElement(View, null,
            React.createElement(View, { style: styles.usernameWrapper },
                React.createElement(LineFormField, { testID: "usernameFormField", onChangeText: this.handleChangeUsername, value: this.props.username, label: s.strings.username, returnKeyType: "next", autoCorrect: false, autoFocus: this.state.focusFirst, forceFocus: this.state.focusFirst, onFocus: this.handleFocus1, onSubmitEditing: this.handleSetNextFocus }),
                React.createElement(TouchableOpacity, { style: styles.iconContainer, onPress: this.handleToggleUsernameList }, this.state.usernameList ? (React.createElement(MaterialIcon, { name: "expand-less", size: theme.rem(1.5), style: styles.iconColor })) : (React.createElement(MaterialIcon, { name: "expand-more", size: theme.rem(1.5), style: styles.iconColor })))),
            !this.state.usernameList ? null : this.renderDropdownList()));
    }
    renderDropdownList() {
        const { theme } = this.props;
        const styles = getStyles(theme);
        return (React.createElement(ScrollView, { style: styles.dropDownList }, this.props.usernameOnlyList.map(item => (React.createElement(UserListItem, { key: item, data: item, onClick: this.handleSelectUser, onDelete: this.handleDelete })))));
    }
    renderButtons() {
        const { handleQrModal, theme } = this.props;
        const styles = getStyles(theme);
        const buttonType = theme.preferPrimaryButton ? 'primary' : 'secondary';
        return (React.createElement(View, { style: styles.buttonsBox },
            React.createElement(MainButton, { type: "textOnly", onPress: this.handleForgotPassword, label: s.strings.forgot_password }),
            React.createElement(View, { style: styles.loginButtonBox },
                React.createElement(MainButton, { label: s.strings.login_button, testID: "loginButton", type: buttonType, onPress: this.handleSubmit })),
            React.createElement(MainButton, { type: "textOnly", testID: "createAccountButton", onPress: this.handleCreateAccount, label: s.strings.create_an_account }),
            React.createElement(TouchableOpacity, { onPress: handleQrModal },
                React.createElement(AntDesignIcon, { name: "qrcode", color: theme.icon, size: theme.rem(1.75) }))));
    }
}
const getStyles = cacheStyles((theme) => ({
    container: {
        flex: 1,
        width: '100%',
        height: '100%',
        backgroundColor: theme.backgroundGradientColors[0]
    },
    mainScrollView: {
        position: 'relative',
        width: '100%',
        height: '100%'
    },
    featureBoxContainer: {
        width: '100%'
    },
    featureBox: {
        position: 'relative',
        top: theme.rem(3.5),
        width: '100%',
        alignItems: 'center'
    },
    shimTiny: {
        width: '100%',
        height: theme.rem(0.75)
    },
    loginButtonBox: {
        marginVertical: theme.rem(0.25),
        width: '70%'
    },
    buttonsBox: {
        width: '100%',
        alignItems: 'center'
    },
    usernameWrapper: {
        width: '100%',
        flexDirection: 'row'
    },
    dropDownList: {
        flexGrow: 0,
        maxHeight: theme.rem(12.5),
        backgroundColor: theme.backgroundGradientColors[0]
    },
    iconContainer: {
        flexDirection: 'row',
        justifyContent: 'center',
        alignItems: 'center',
        position: 'absolute',
        right: theme.rem(-0.5),
        width: theme.rem(2),
        height: theme.rem(2),
        bottom: theme.rem(0.5)
    },
    iconColor: {
        color: theme.icon
    },
    iconColorPressed: {
        color: theme.iconDeactivated
    }
}));
export function PasswordLoginScene(props) {
    const { branding, route } = props;
    const dispatch = useDispatch();
    const theme = useTheme();
    const loginSuccess = useSelector(state => state.login.loginSuccess);
    const previousUsers = useSelector(state => state.previousUsers.userList);
    const touch = useSelector(state => state.touch.type);
    const username = useSelector(state => state.login.username);
    const usernameOnlyList = useSelector(state => state.previousUsers.usernameOnlyList);
    const dispatchProps = {
        deleteUserFromDevice(username) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield dispatch(deleteUserFromDevice(username));
            });
        },
        gotoCreatePage() {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'newAccountWelcome', params: {} }
            });
        },
        gotoPinLoginPage() {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'pinLogin', params: {} }
            });
        },
        handleQrModal() {
            Keyboard.dismiss();
            dispatch(showQrCodeModal());
        },
        login(attempt) {
            return __awaiter(this, void 0, void 0, function* () {
                return yield dispatch(login(attempt));
            });
        },
        exitScene() {
            dispatch(maybeRouteComplete({
                type: 'NAVIGATE',
                data: { name: 'landing', params: {} }
            }));
        },
        saveOtpError(otpAttempt, otpError) {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'otpError', params: { otpAttempt, otpError } }
            });
        },
        updateUsername(data) {
            dispatch({ type: 'AUTH_UPDATE_USERNAME', data: data });
        },
        handleSubmitRecoveryKey(recoveryKey) {
            var _a;
            return __awaiter(this, void 0, void 0, function* () {
                if (((_a = base58.parseUnsafe(recoveryKey)) === null || _a === void 0 ? void 0 : _a.length) !== 32)
                    return s.strings.recovery_token_invalid;
                dispatch(launchPasswordRecovery(recoveryKey));
                return yield Promise.resolve(true);
            });
        }
    };
    return (React.createElement(PasswordLoginSceneComponent, Object.assign({}, dispatchProps, { branding: branding, loginSuccess: loginSuccess, previousUsers: previousUsers, route: route, theme: theme, touch: touch, username: username, usernameOnlyList: usernameOnlyList })));
}
