var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as React from 'react';
import { FlatList, Keyboard, Platform, Text, TouchableOpacity, TouchableWithoutFeedback, View } from 'react-native';
import LinearGradient from 'react-native-linear-gradient';
import { cacheStyles } from 'react-native-patina';
import { SafeAreaView } from 'react-native-safe-area-context';
import { SvgXml } from 'react-native-svg';
import MaterialCommunityIcons from 'react-native-vector-icons/MaterialCommunityIcons';
import { sprintf } from 'sprintf-js';
import { loginWithPin, loginWithTouch } from '../../actions/LoginAction';
import { deleteUserFromDevice } from '../../actions/UserActions';
import { FaceIdXml } from '../../assets/xml/FaceId';
import s from '../../common/locales/strings';
import { useDispatch, useSelector } from '../../types/ReduxTypes';
import { FourDigit } from '../abSpecific/FourDigitComponent';
import { LogoImageHeader } from '../abSpecific/LogoImageHeader';
import { PinKeypad } from '../abSpecific/PinKeypad';
import { UserListItem } from '../abSpecific/UserListItem';
import { ButtonsModal } from '../modals/ButtonsModal';
import { Airship, showError } from '../services/AirshipInstance';
import { useTheme } from '../services/ThemeContext';
import { ThemedScene } from '../themed/ThemedScene';
export function PinLoginScene(props) {
    const { branding } = props;
    const dispatch = useDispatch();
    const theme = useTheme();
    const styles = getStyles(theme);
    // ---------------------------------------------------------------------
    // State
    // ---------------------------------------------------------------------
    const [focusOn, setFocusOn] = React.useState('pin');
    // Pin login state:
    const errorMessage = useSelector(state => state.login.errorMessage || '');
    const isLoggingInWithPin = useSelector(state => state.login.isLoggingInWithPin);
    const loginSuccess = useSelector(state => state.login.loginSuccess);
    const pin = useSelector(state => state.login.pin || '');
    const touch = useSelector(state => state.touch.type);
    const wait = useSelector(state => state.login.wait);
    const isTouchIdDisabled = loginSuccess || !!wait || isLoggingInWithPin || pin.length === 4;
    // User state:
    const userList = useSelector(state => state.previousUsers.userList);
    const username = useSelector(state => state.login.username);
    const userDetails = React.useMemo(() => {
        var _a;
        return (_a = userList.find(user => user.username === username)) !== null && _a !== void 0 ? _a : {
            username,
            pinEnabled: false,
            touchEnabled: false
        };
    }, [userList, username]);
    const dropdownItems = React.useMemo(() => userList
        .filter(user => user.pinEnabled || (touch && user.touchEnabled))
        .map(user => user.username), [touch, userList]);
    // ---------------------------------------------------------------------
    // Effects
    // ---------------------------------------------------------------------
    // Runs once at start:
    React.useEffect(() => {
        if (username && touch !== 'FaceID') {
            dispatch(loginWithTouch(username)).catch(showError);
        }
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    React.useEffect(() => {
        if (!userDetails.touchEnabled && !userDetails.pinEnabled) {
            dispatch({
                type: 'NAVIGATE',
                data: { name: 'passwordLogin', params: {} }
            });
        }
    }, [dispatch, userDetails]);
    // ---------------------------------------------------------------------
    // Handlers
    // ---------------------------------------------------------------------
    const handleBack = () => {
        dispatch({
            type: 'NAVIGATE',
            data: { name: 'passwordLogin', params: {} }
        });
    };
    const handleDelete = (username) => {
        setFocusOn('pin');
        Keyboard.dismiss();
        Airship.show(bridge => (React.createElement(ButtonsModal, { bridge: bridge, title: s.strings.forget_account, message: sprintf(s.strings.forget_username_account, username), buttons: {
                ok: { label: s.strings.forget },
                cancel: { label: s.strings.cancel, type: 'secondary' }
            } })))
            .then((button) => __awaiter(this, void 0, void 0, function* () {
            if (button !== 'ok')
                return;
            return yield dispatch(deleteUserFromDevice(username));
        }))
            .catch(showError);
    };
    const handleTouchId = () => {
        dispatch(loginWithTouch(username)).catch(showError);
    };
    const handlePress = (value) => {
        const newPin = value === 'back' ? pin.slice(0, -1) : pin.concat(value).slice(0, 4);
        if (newPin.length === 4 && pin.length === 3) {
            dispatch(loginWithPin(username, newPin));
        }
        dispatch({ type: 'AUTH_UPDATE_PIN', data: newPin });
    };
    const handleSelectUser = (username) => {
        dispatch(loginWithTouch(username)).catch(showError);
        dispatch({ type: 'AUTH_UPDATE_USERNAME', data: username });
        setFocusOn('pin');
    };
    const handleShowDrop = () => {
        setFocusOn('List');
    };
    const handleHideDrop = () => {
        setFocusOn('pin');
    };
    // ---------------------------------------------------------------------
    // Rendering
    // ---------------------------------------------------------------------
    const renderOverImage = () => {
        if (loginSuccess) {
            return null;
        }
        return (React.createElement(View, { style: styles.featureBoxContainer },
            React.createElement(TouchableWithoutFeedback, { onPress: handleHideDrop },
                React.createElement(View, { style: styles.featureBox },
                    React.createElement(LogoImageHeader, { branding: branding }),
                    React.createElement(View, { style: styles.featureBoxBody }, renderBottomHalf()))),
            React.createElement(View, { style: styles.spacer_full }),
            !userDetails.pinEnabled ? null : (React.createElement(PinKeypad, { disabled: wait > 0 || pin.length === 4, onPress: handlePress })),
            React.createElement(SafeAreaView, { edges: ['bottom'] })));
    };
    const renderBottomHalf = () => {
        if (focusOn === 'pin') {
            return (React.createElement(View, { style: styles.innerView },
                React.createElement(TouchableOpacity, { style: styles.usernameShadow, onPress: handleShowDrop },
                    React.createElement(LinearGradient, { colors: theme.pinUsernameButton, start: theme.pinUsernameButtonColorStart, end: theme.pinUsernameButtonColorEnd, style: [styles.linearGradient, styles.usernameButton] },
                        React.createElement(Text, { adjustsFontSizeToFit: true, minimumFontScale: 0.75, numberOfLines: 1, style: styles.usernameText }, username))),
                !userDetails.pinEnabled ? null : (React.createElement(FourDigit, { error: wait > 0
                        ? `${errorMessage}: ${sprintf(s.strings.account_locked_for, wait)}`
                        : errorMessage, pin: pin, spinner: wait > 0 || pin.length === 4 || isLoggingInWithPin })),
                userDetails.pinEnabled ? null : React.createElement(View, { style: styles.spacer }),
                renderTouchImage(),
                React.createElement(Text, { style: styles.touchImageText }, renderTouchImageText())));
        }
        return (React.createElement(View, { style: styles.innerView },
            React.createElement(FlatList, { style: styles.listView, data: dropdownItems, renderItem: renderItems, keyExtractor: (item, index) => index.toString() })));
    };
    const renderItems = (item) => {
        return (React.createElement(UserListItem, { data: item.item, onClick: handleSelectUser, onDelete: handleDelete }));
    };
    const renderTouchImage = () => {
        const { touchEnabled } = userDetails;
        if (touchEnabled && touch === 'FaceID') {
            return (React.createElement(TouchableOpacity, { onPress: handleTouchId, disabled: isTouchIdDisabled },
                React.createElement(SvgXml, { xml: FaceIdXml, color: theme.iconTappable, width: theme.rem(3), height: theme.rem(3) })));
        }
        if (touchEnabled && touch === 'TouchID') {
            return (React.createElement(TouchableOpacity, { onPress: handleTouchId, disabled: isTouchIdDisabled },
                React.createElement(MaterialCommunityIcons, { name: "fingerprint", size: theme.rem(3), color: theme.iconTappable })));
        }
        if (!touchEnabled || !touch) {
            return null;
        }
        return null;
    };
    const renderTouchImageText = () => {
        const { touchEnabled } = userDetails;
        if (touchEnabled && touch === 'FaceID') {
            return s.strings.use_faceId;
        }
        if (touchEnabled && touch === 'TouchID' && Platform.OS === 'ios') {
            return s.strings.use_touchId;
        }
        if (touchEnabled && touch === 'TouchID' && Platform.OS !== 'ios') {
            return s.strings.use_fingerprint;
        }
        return '';
    };
    return (React.createElement(ThemedScene, { backButtonText: s.strings.exit_pin, onBack: handleBack, noUnderline: true, branding: branding },
        React.createElement(View, { style: styles.container }, renderOverImage())));
}
const getStyles = cacheStyles((theme) => ({
    container: {
        flex: 1,
        width: '100%',
        height: '100%'
    },
    listView: {
        height: theme.rem(16),
        width: theme.rem(10)
    },
    featureBoxContainer: {
        width: '100%',
        flex: 1,
        flexDirection: 'column',
        justifyContent: 'space-around'
    },
    featureBox: {
        width: '100%',
        alignItems: 'center'
    },
    featureBoxBody: {
        height: theme.rem(15),
        width: '100%'
    },
    innerView: {
        height: '100%',
        width: '100%',
        justifyContent: 'flex-start',
        alignItems: 'center'
    },
    touchImageText: {
        marginTop: theme.rem(0.5),
        color: theme.iconTappable
    },
    usernameText: {
        fontFamily: theme.pinUsernameButtonFont,
        color: theme.pinUsernameButtonText,
        fontSize: theme.rem(theme.pinUsernameButtonFontSizeRem),
        paddingHorizontal: theme.rem(1),
        paddingVertical: theme.rem(0.5)
    },
    usernameButton: {
        borderColor: theme.pinUsernameButtonOutline,
        borderWidth: theme.pinUsernameButtonOutlineWidth
    },
    usernameShadow: Object.assign({}, theme.pinUsernameButtonShadow),
    linearGradient: {
        // flex: 1,
        paddingLeft: 0,
        paddingRight: 0,
        borderRadius: theme.rem(theme.pinUsernameButtonBorderRadiusRem)
    },
    spacer: {
        marginTop: theme.rem(2)
    },
    spacer_full: {
        flex: 1,
        zIndex: -100
    }
}));
