import { asArray, asBoolean, asNumber, asObject, asOptional, asString, asValue } from 'cleaners';
const asFunction = raw => {
    if (typeof raw === 'function')
        return raw;
    throw new TypeError();
};
const asThemeShadowParams = asObject({
    shadowColor: asString,
    shadowOffset: asObject({
        width: asNumber,
        height: asNumber
    }),
    shadowOpacity: asNumber,
    shadowRadius: asNumber,
    elevation: asNumber
});
const asTextShadowParams = asObject({
    textShadowColor: asString,
    textShadowOffset: asObject({
        width: asNumber,
        height: asNumber
    }),
    textShadowRadius: asNumber
});
const asGradientCoords = asObject({
    x: asNumber,
    y: asNumber
});
export const themeNoShadow = {
    shadowColor: '#000000',
    shadowOffset: {
        width: 0,
        height: 0
    },
    shadowOpacity: 0,
    shadowRadius: 0,
    elevation: 0
};
export const textNoShadow = {
    textShadowColor: '#000000',
    textShadowOffset: {
        width: 0,
        height: 0
    },
    textShadowRadius: 0
};
export const asOptionalTheme = asObject({
    // The app scaling factor, which is the height of "normal" text:
    rem: asOptional(asFunction),
    preferPrimaryButton: asOptional(asBoolean),
    fontFamily: asOptional(asString),
    fontWeightBold: asOptional(asValue('normal', 'bold', '100', '200', '300', '400', '500', '600', '700', '800', '900')),
    // Icons
    icon: asOptional(asString),
    iconDeactivated: asOptional(asString),
    iconTappable: asOptional(asString),
    // Background colors:
    backgroundGradientColors: asOptional(asArray(asString)),
    backgroundGradientStart: asOptional(asObject({ x: asNumber, y: asNumber })),
    backgroundGradientEnd: asOptional(asObject({ x: asNumber, y: asNumber })),
    // Modal:
    modal: asOptional(asString),
    modalBlurType: asOptional(asValue('light', 'dark')),
    modalBorderColor: asOptional(asString),
    modalBorderWidth: asOptional(asNumber),
    modalBorderRadiusRem: asOptional(asNumber),
    // Text colors:
    primaryText: asOptional(asString),
    secondaryText: asOptional(asString),
    dangerText: asOptional(asString),
    warningText: asOptional(asString),
    linkText: asOptional(asString),
    positiveText: asOptional(asString),
    deactivatedText: asOptional(asString),
    // Tile:
    tileBackground: asOptional(asString),
    // Buttons
    buttonBorderRadiusRem: asOptional(asNumber),
    keypadButtonOutline: asOptional(asString),
    keypadButtonOutlineWidth: asOptional(asNumber),
    keypadButton: asOptional(asArray(asString)),
    keypadButtonColorStart: asOptional(asGradientCoords),
    keypadButtonColorEnd: asOptional(asGradientCoords),
    keypadButtonText: asOptional(asString),
    keypadButtonTextShadow: asOptional(asTextShadowParams),
    keypadButtonShadow: asOptional(asThemeShadowParams),
    keypadButtonBorderRadiusRem: asOptional(asNumber),
    keypadButtonFontSizeRem: asOptional(asNumber),
    keypadButtonFont: asOptional(asString),
    primaryButtonOutline: asOptional(asString),
    primaryButtonOutlineWidth: asOptional(asNumber),
    primaryButton: asOptional(asArray(asString)),
    primaryButtonColorStart: asOptional(asGradientCoords),
    primaryButtonColorEnd: asOptional(asGradientCoords),
    primaryButtonText: asOptional(asString),
    primaryButtonTextShadow: asOptional(asTextShadowParams),
    primaryButtonShadow: asOptional(asThemeShadowParams),
    primaryButtonFontSizeRem: asOptional(asNumber),
    primaryButtonFont: asOptional(asString),
    secondaryButtonOutline: asOptional(asString),
    secondaryButtonOutlineWidth: asOptional(asNumber),
    secondaryButton: asOptional(asArray(asString)),
    secondaryButtonColorStart: asOptional(asGradientCoords),
    secondaryButtonColorEnd: asOptional(asGradientCoords),
    secondaryButtonText: asOptional(asString),
    secondaryButtonTextShadow: asOptional(asTextShadowParams),
    secondaryButtonShadow: asOptional(asThemeShadowParams),
    secondaryButtonFontSizeRem: asOptional(asNumber),
    secondaryButtonFont: asOptional(asString),
    escapeButtonOutline: asOptional(asString),
    escapeButtonOutlineWidth: asOptional(asNumber),
    escapeButton: asOptional(asArray(asString)),
    escapeButtonColorStart: asOptional(asGradientCoords),
    escapeButtonColorEnd: asOptional(asGradientCoords),
    escapeButtonText: asOptional(asString),
    escapeButtonTextShadow: asOptional(asTextShadowParams),
    escapeButtonShadow: asOptional(asThemeShadowParams),
    escapeButtonFontSizeRem: asOptional(asNumber),
    escapeButtonFont: asOptional(asString),
    pinUsernameButtonOutline: asOptional(asString),
    pinUsernameButtonOutlineWidth: asOptional(asNumber),
    pinUsernameButton: asOptional(asArray(asString)),
    pinUsernameButtonColorStart: asOptional(asGradientCoords),
    pinUsernameButtonColorEnd: asOptional(asGradientCoords),
    pinUsernameButtonText: asOptional(asString),
    pinUsernameButtonTextShadow: asOptional(asTextShadowParams),
    pinUsernameButtonShadow: asOptional(asThemeShadowParams),
    pinUsernameButtonBorderRadiusRem: asOptional(asNumber),
    pinUsernameButtonFontSizeRem: asOptional(asNumber),
    pinUsernameButtonFont: asOptional(asString),
    // Outline Text Input
    outlineTextInputColor: asOptional(asString),
    outlineTextInputTextColor: asOptional(asString),
    outlineTextInputBorderWidth: asOptional(asNumber),
    outlineTextInputBorderColor: asOptional(asString),
    outlineTextInputBorderColorFocused: asOptional(asString),
    outlineTextInputLabelColor: asOptional(asString),
    outlineTextInputLabelColorFocused: asOptional(asString),
    // Dropdown colors:
    dropdownWarning: asOptional(asString),
    dropdownError: asOptional(asString),
    dropdownText: asOptional(asString),
    // Native iOS date modal:
    dateModalTextLight: asOptional(asString),
    dateModalTextDark: asOptional(asString),
    dateModalBackgroundLight: asOptional(asString),
    dateModalBackgroundDark: asOptional(asString),
    // Card
    cardBorder: asOptional(asNumber),
    cardBorderColor: asOptional(asString),
    cardBorderRadius: asOptional(asNumber),
    // Lines
    lineDivider: asOptional(asString),
    thinLineWidth: asOptional(asNumber),
    mediumLineWidth: asOptional(asNumber),
    // Font
    fontFaceDefault: asOptional(asString),
    fontFaceMedium: asOptional(asString),
    fontFaceBold: asOptional(asString),
    fontFaceSymbols: asOptional(asString)
});
